@@ -17,23 +17,25 @@ from attr import Attribute
 from attr._make import NOTHING, make_class
 
 
-def simple_class(cmp=False, repr=False, hash=False, slots=False):
+def simple_class(cmp=False, repr=False, hash=False, str=False, slots=False,
+                 frozen=False):
     """
     Return a new simple class.
     """
     return make_class(
         "C", ["a", "b"],
-        cmp=cmp, repr=repr, hash=hash, init=True, slots=slots
+        cmp=cmp, repr=repr, hash=hash, init=True, slots=slots, str=str,
+        frozen=frozen,
     )
 
 
 def simple_attr(name, default=NOTHING, validator=None, repr=True,
-                cmp=True, hash=True, init=True):
+                cmp=True, hash=None, init=True):
     """
     Return an attribute with a name and no other bells and whistles.
     """
     return Attribute(
-        name=name, default=default, validator=validator, repr=repr,
+        name=name, _default=default, _validator=validator, repr=repr,
         cmp=cmp, hash=hash, init=init
     )
 
@@ -55,7 +57,7 @@ class TestSimpleClass(object):
         assert simple_class() is not simple_class()
 
 
-def _gen_attr_names():
+def gen_attr_names():
     """
     Generate names for attributes, 'a'...'z', then 'aa'...'zz'.
 
@@ -74,11 +76,23 @@ def _gen_attr_names():
             yield outer + inner
 
 
+def maybe_underscore_prefix(source):
+    """
+    A generator to sometimes prepend an underscore.
+    """
+    to_underscore = False
+    for val in source:
+        yield val if not to_underscore else '_' + val
+        to_underscore = not to_underscore
+
+
 def _create_hyp_class(attrs):
     """
     A helper function for Hypothesis to generate attrs classes.
     """
-    return make_class('HypClass', dict(zip(_gen_attr_names(), attrs)))
+    return make_class(
+        "HypClass", dict(zip(gen_attr_names(), attrs))
+    )
 
 
 def _create_hyp_nested_strategy(simple_class_strategy):
@@ -130,6 +144,7 @@ def _create_hyp_nested_strategy(simple_class_strategy):
                      attrs_and_classes.map(dict_of_class),
                      attrs_and_classes.map(ordereddict_of_class))
 
+
 bare_attrs = st.just(attr.ib(default=None))
 int_attrs = st.integers().map(lambda i: attr.ib(default=i))
 str_attrs = st.text().map(lambda s: attr.ib(default=s))
@@ -137,40 +152,78 @@ float_attrs = st.floats().map(lambda f: attr.ib(default=f))
 dict_attrs = (st.dictionaries(keys=st.text(), values=st.integers())
               .map(lambda d: attr.ib(default=d)))
 
-simple_attrs = st.one_of(bare_attrs, int_attrs, str_attrs, float_attrs,
-                         dict_attrs)
-
-# Python functions support up to 255 arguments.
-list_of_attrs = st.lists(simple_attrs, average_size=9, max_size=50)
+simple_attrs_without_metadata = (bare_attrs | int_attrs | str_attrs |
+                                 float_attrs | dict_attrs)
 
 
 @st.composite
-def simple_classes(draw, slots=None, frozen=None):
-    """A strategy that generates classes with default non-attr attributes.
+def simple_attrs_with_metadata(draw):
+    """
+    Create a simple attribute with arbitrary metadata.
+    """
+    c_attr = draw(simple_attrs)
+    keys = st.booleans() | st.binary() | st.integers() | st.text()
+    vals = st.booleans() | st.binary() | st.integers() | st.text()
+    metadata = draw(st.dictionaries(keys=keys, values=vals))
+
+    return attr.ib(c_attr.default, c_attr._validator, c_attr.repr,
+                   c_attr.cmp, c_attr.hash, c_attr.init, c_attr.convert,
+                   metadata)
+
+
+simple_attrs = simple_attrs_without_metadata | simple_attrs_with_metadata()
+
+# Python functions support up to 255 arguments.
+list_of_attrs = st.lists(simple_attrs, average_size=3, max_size=9)
+
+
+@st.composite
+def simple_classes(draw, slots=None, frozen=None, private_attrs=None):
+    """
+    A strategy that generates classes with default non-attr attributes.
 
     For example, this strategy might generate a class such as:
 
     @attr.s(slots=True, frozen=True)
     class HypClass:
         a = attr.ib(default=1)
-        b = attr.ib(default=None)
+        _b = attr.ib(default=None)
         c = attr.ib(default='text')
-        d = attr.ib(default=1.0)
+        _d = attr.ib(default=1.0)
         c = attr.ib(default={'t': 1})
 
     By default, all combinations of slots and frozen classes will be generated.
     If `slots=True` is passed in, only slots classes will be generated, and
     if `slots=False` is passed in, no slot classes will be generated. The same
     applies to `frozen`.
+
+    By default, some attributes will be private (i.e. prefixed with an
+    underscore). If `private_attrs=True` is passed in, all attributes will be
+    private, and if `private_attrs=False`, no attributes will be private.
     """
     attrs = draw(list_of_attrs)
     frozen_flag = draw(st.booleans()) if frozen is None else frozen
     slots_flag = draw(st.booleans()) if slots is None else slots
 
-    return make_class('HypClass', dict(zip(_gen_attr_names(), attrs)),
+    if private_attrs is None:
+        attr_names = maybe_underscore_prefix(gen_attr_names())
+    elif private_attrs is True:
+        attr_names = ('_' + n for n in gen_attr_names())
+    elif private_attrs is False:
+        attr_names = gen_attr_names()
+
+    cls_dict = dict(zip(attr_names, attrs))
+    post_init_flag = draw(st.booleans())
+    if post_init_flag:
+        def post_init(self):
+            pass
+        cls_dict["__attrs_post_init__"] = post_init
+    return make_class("HypClass", cls_dict,
                       slots=slots_flag, frozen=frozen_flag)
 
-# Ok, so st.recursive works by taking a base strategy (in this case,
-# simple_classes) and a special function. This function receives a strategy,
-# and returns another strategy (building on top of the base strategy).
-nested_classes = st.recursive(simple_classes(), _create_hyp_nested_strategy)
+
+# st.recursive works by taking a base strategy (in this case, simple_classes)
+# and a special function.  This function receives a strategy, and returns
+# another strategy (building on top of the base strategy).
+nested_classes = st.recursive(simple_classes(), _create_hyp_nested_strategy,
+                              max_leaves=10)
